; Copyright 2018-2019 Sean Porter (Glitchfinder)
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;------------------------------------------------------------------------------/

Scriptname zzGPatrolMaintenance extends Quest

; The settlement manager script
WorkshopParentScript Property Manager auto const mandatory

ActorBase Property WorkshopNPC Auto ; The base actor to spawn
Faction   Property Minutemen   Auto ; A custom minutemen faction
Keyword   Property Follow2     Auto ; A keyword for setting patrol leaders
Quest     Property TFSQuest    Auto ; The First Steps quest
Int       Property Number      Auto ; The number of people in a patrol. 1-3

WorkshopScript[] Property Workshops       Auto Hidden ; All tracked workshops
WorkshopScript   Property CurrentLocation Auto Hidden ; Workshop currently spawning
Keyword          Property Follow          Auto Hidden ; Primary patrol leader keyword
Keyword          Property Center          Auto Hidden ; Workshop location keyword
Keyword          Property WStart          Auto Hidden ; Path start keyword
Keyword          Property WEnd            Auto Hidden ; Path end keyword
Quest            Property HSHQuest        Auto Hidden ; Home Sweet Home quest
Quest            Property OSQuest         Auto Hidden ; Open Season quest
Bool             Property Spawning        Auto Hidden ; Are patrols spawning?
Bool             Property NukaWorld       Auto Hidden ; Is Nuka World installed?
Bool             Property Active          Auto Hidden ; Is this mod active?
Int              Property CurrentCount    Auto Hidden ; Number of patrols spawning
Int              Property Index           Auto Hidden ; Current settlement index
Int              Property NukaCheck       Auto Hidden ; Countdown for DLC check
Int              Property Food            Auto Hidden ; Food value index
Int              Property Water           Auto Hidden ; Water value index
Int              Property Safety          Auto Hidden ; Safety value index

; Start the mod
Event OnQuestInit()
	; Grab keywords and indices from the workshp manager
	Follow = Manager.WorkshopLinkFollow
	Center = Manager.WorkshopLinkCenter
	WStart = Manager.WorkshopLinkCaravanStart
	WEnd   = Manager.WorkshopLinkCaravanEnd
	Food   = Manager.WorkshopRatingFood
	Water  = Manager.WorkshopRatingWater
	Safety = Manager.WorkshopRatingSafety

	; Default tracked data
	Active = False
	Spawning = False
	CurrentCount = 0
	Index = 0
	Workshops = New WorkshopScript[100]
	NukaCheck = 0
	CheckDLC()

	; Start the mod's timer loop
	StartTimerGameTime(0.033)
EndEvent

; Check for changes
Event OnTimerGameTime(Int aiTimerID)
	; Ensure this script only runs on objects that exist
	If (!IsBoundGameObjectAvailable())
		Return
	EndIf

	; If the mod is not active
	If !Active
		; Check the minutemen quest status
		Active = TFSQuest.GetStage() >= 450
		; Defer further processing until later
		StartTimerGameTime(0.033)
		Return
	EndIf

	; If a valid settlement is spawning patrols
	If Spawning
		; Handle patrol spawning
		CheckNPCs()
	; If no valid settlements are spawning patrols
	Else
		; Check for a valid settlement
		CheckWorkshops()
	EndIf

	; If Nuka World was not installed
	If (NukaWorld != True)
		; Check to see if it is installed now
		CheckDLC()
	; If Nuka World is installed
	Else
		; Check faction hostility
		CheckAlliance()
	EndIf

	; Restart and defer these checks
	StartTimerGameTime(0.033)
EndEvent

; Check to see if Nuka World is installed
Function CheckDLC()
	; Make sure this check doesn't run constantly
	If (NukaCheck > 0)
		NukaCheck -= 1
		Return
	EndIf

	; Try to load the Home Sweet Home quest from file
	HSHQuest = Game.GetFormFromFile(0x00000804, "DLCNukaWorld.esm") as Quest
	; Set the Nuka World installation status based on whether it loaded
	NukaWorld = (HSHQuest != None)

	; If Nuka World is not installed, prevent another check for a while
	If (NukaWorld != True)
		NukaCheck = 50
	; If Nuka World is installed, load the Open Season quest from file
	Else
		OSQuest  = Game.GetFormFromFile(0x00027704, "DLCNukaWorld.esm") as Quest
	EndIf
EndFunction

; Check to see if the minutemen are hostile
Function CheckAlliance()
	; If Home Sweet Home is incomplete, they are not hostile
	If (HSHQuest.GetStage() < 1000)
		Return
	EndIf

	; If Open Season is complete, they are not hostile
	If (OSQuest.GetStage() >= 255)
		Return
	EndIf

	; If Home Sweet Home is complete and Open Season is not, they are hostile
	Minutemen.SetPlayerEnemy(True)
EndFunction

; Check for new workshops to spawn patrols at
Function CheckWorkshops()
	; Get the number of workshops in the game
	Int i = Manager.Workshops.Length

	; Ensure this is not greater than 100
	If (i > 100)
		i = 100
	EndIf

	; Go through every workshop in the list, up to 100
	While i
		; Lower the index by one
		i -= 1

		; Get the current workshop to check
		WorkshopScript w = Manager.Workshops[i] As WorkshopScript

		; If the workshop has not already been handled
		If Workshops.Find(w) < 0
			; Set up a check for validity
			Bool skip = False;

			; The workshop is invalid if it is actually a raider outpost
			If (w.HasKeyword(Manager.WorkshopType02) == True)
				skip = True;
			EndIf

			; Set up a production value check
			Int value = 0

			; Get the settlement's current production values
			value += (w.GetValue(Manager.WorkshopRatings[Food].resourceValue) as Int)
			value += (w.GetValue(Manager.WorkshopRatings[Water].resourceValue) as Int)
			value += (w.GetValue(Manager.WorkshopRatings[Safety].resourceValue) as Int)

			; If the workshop is player owned, not a raider camp, and produces something
			If (w.OwnedByPlayer && !skip && value > 0)
				; Add it to the list of checked workshops
				Workshops[i] = w

				; If there is already at least one tracked settlement
				If (CurrentCount > 0)
					; Start spawning patrols instead of checking settlements
					Spawning = True
					; Set the workshop check index
					Index = 100
					; Cache the current workshop for spawning
					CurrentLocation = w
				EndIf

				; Increase the number of tracked settlements
				CurrentCount += 1
				; Stop checking for new settlements
				i = 0
			EndIf
		; If the workshop is already handled
		Else
			; If it has turned into a raider outpost
			If (w.HasKeyword(Manager.WorkshopType02) == True)
				; Remove it from the list of tracked settlements
				Workshops[i] = None
				CurrentCount -= 1
			EndIf
		EndIf
	EndWhile
EndFunction

; Handle patrol spawning
Function CheckNPCs()
	; Spawn up to five patrols per attempt, to prevent overload
	Int count = 5

	; Start attempting to spawn patrols
	While count > 0
		; Attempt to spawn a patrol
		FindPatrolTarget()

		; If all patrols have been spawned
		If Index <= 0
			; Reset back to settlement checking
			Spawning = False
			Index = 0
			Return
		EndIf

		; Reduce the number of remaining spawn attempts
		count -= 1
	EndWhile
EndFunction

; Attempt to spawn a patrol
Function FindPatrolTarget()
	; Run through the list of settlements
	While Index
		; Move to the next settlement index
		Index -= 1
		; Grab the settlement in question
		WorkshopScript c = Workshops[Index]

		; If the settlement exists and is not the one being processed
		If (c != None && c != CurrentLocation)
			; Spawn a patrol
			SpawnNPC()
			Return
		EndIf
	EndWhile
EndFunction

; Spawn a single patrol
Function SpawnNPC()
	; Set up the destination reference
	ObjectReference ref

	; If the normal destination is currently loaded
	If (Workshops[Index].Is3DLoaded())
		; Set the destination to the current workshop
		ref = CurrentLocation.GetLinkedRef(Center)
	; If the normal destination is not loaded
	Else
		; Set the destination to the other workshop
		ref = Workshops[Index].GetLinkedRef(Center)
	EndIf

	; Place a new patroller
	Actor newActor = ref.PlaceActorAtMe(WorkshopNPC, 0, None)

	; Set the patroller's destinations
	zzGPatrollerScript a = newActor As zzGPatrollerScript
	WorkshopScript workshopDestination = Workshops[Index]
	WorkshopScript workshopStart = CurrentLocation

	; Set the destination endpoints
	a.SetLinkedRef(workshopStart.GetLinkedRef(Center), WStart)
	a.SetLinkedRef(workshopDestination.GetLinkedRef(Center), WEnd)

	; Spawn a second patroller if the patrol has more than one person
	If (Number > 1)
		a.Follow1 = a.PlaceActorAtMe(WorkshopNPC, 0, None) As Actor
		a.Follow1.SetLinkedRef(a, Follow)
		a.Follow1.SetLinkedRef(workshopStart.GetLinkedRef(Center), WStart)
		a.Follow1.SetLinkedRef(workshopDestination.GetLinkedRef(Center), WEnd)
		(a.Follow1 As zzGPatrollerScript).Init2()
	EndIf

	; Spawn a third patroller if the patrol has three people
	If (Number > 2)
		a.Follow2 = a.PlaceActorAtMe(WorkshopNPC, 0, None) As Actor
		a.Follow2.SetLinkedRef(a, Follow)
		a.Follow2.SetLinkedRef(a.Follow1 as zzGPatrollerScript, Follow2)
		a.Follow2.SetLinkedRef(workshopStart.GetLinkedRef(Center), WStart)
		a.Follow2.SetLinkedRef(workshopDestination.GetLinkedRef(Center), WEnd)
		(a.Follow2 As zzGPatrollerScript).Init2()
	EndIf

	; Initialize the lead patroller
	(a As zzGPatrollerScript).Init(workshopDestination)
EndFunction