; Copyright 2018 Sean Porter (Glitchfinder)
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;------------------------------------------------------------------------------/

Scriptname zzGPatrollerScript extends Actor

; The two other patrollers
Actor Property Follow1 Auto Hidden
Actor Property Follow2 Auto Hidden

; The Minutemen Faction
Faction Property Minutemen Auto

; The actor's position for testing the patrol path
Float Property CX Auto Hidden
Float Property CY Auto Hidden
Float Property CZ Auto Hidden

; The actor's position for reset handling
Float Property RX Auto Hidden
Float Property RY Auto Hidden
Float Property RZ Auto Hidden

; Double-test flag, in case an actor unloads after being jostled
Bool Property Double Auto Hidden

; The patrol destination
ObjectReference Property TestTarget Auto Hidden

; Patrol test function
Function Init(ObjectReference Target)
	; Cache the patrol destination and actor position
	TestTarget = Target
	CX = X
	CY = Y
	CZ = Z

	; Set the double-test flag to false, as the actor has not been checked
	Double = False

	; Defer the actual test so the actor has a chance to move and code
	; doesn't get locked by Actor.PathToReference
	StartTimerGameTime(0.2, 1)

	; Start the NPC reset timer, to stay scaled with the player
	StartTimerGameTime(Utility.RandomInt(1, 7) * 24, 2)
EndFunction

; Initialize patrol followers
Function Init2()
	; Start the NPC reset timer, to stay scaled with the player
	StartTimerGameTime(Utility.RandomInt(1, 7) * 24, 2)
EndFunction

; Receive Timed Events
Event OnTimerGameTime(Int aiTimerID)
	; Ensure this script only runs on objects that exist
	If (!IsBoundGameObjectAvailable())
		Return
	EndIf

	; Handle patrol route tests
	If (aiTimerID == 1)
		TestPatrolRoute()
	EndIf

	; Handle NPC resets
	If (aiTimerID == 2)
		ResetNPC()
	EndIf
EndEvent

; Handle Patroller Death
Event OnDeath(Actor killer)
	; If the player is an enemy of the Minutemen
	If (Minutemen.IsPlayerEnemy() == True)
		; Ignore the patroller's death
		Return
	EndIf

	; Assume the character's death is unintentional and reset them.
	Reset()
EndEvent

; Handle Nuka World Faction Switch
Event OnCombatStateChanged(Actor akTarget, Int aeCombatState)
	; Ensure the patroller is properly killable when hostile
	SetProtected(!Minutemen.IsPlayerEnemy())
EndEvent

; Reset the actor to ensure they stay in line with the player
Function ResetNPC()
	; Check to see if the actor is loaded. This shouldn't happen in view.
	If (Is3DLoaded())
		; Defer the actor reset by one day
		StartTimerGameTime(24, 2)
		Return
	EndIf

	; Cache the actor's current position
	RX = X
	RY = Y
	RZ = Z

	; Reset the actor
	Reset()

	; Fix the actor's position
	SetPosition(RX, RY, RZ)

	; Restart the reset timer
	StartTimerGameTime(168, 2)
EndFunction

; Deferred patrol route test
Function TestPatrolRoute()
	; Check to see if the actor is loaded. There are too many cases to test
	; for if the actor is loaded, and so the check should be deferred again.
	If (Is3DLoaded())
		; Cache the actor's current position
		CX = X
		CY = Y
		CZ = Z

		; If the actor was previously not loaded
		If (Double == True)
			; Unset the double-test flag
			Double = False;
		EndIf

		; Defer the test again.
		StartTimerGameTime(0.2, 1)
		Return
	EndIf

	; If the actor is not loaded but has moved, then they are likely already
	; patrolling as desired and do not need to be tested.
	If (X != cx || Y != cy || Z != cz)
		; If the actor has not been checked twice, they might have been
		; jostled by another actor prior to being unloaded.
		If (Double == False)
			; Cache the actor's current position
			CX = X
			CY = Y
			CZ = Z

			; Set the double-test flag
			Double = True;

			; Defer the test again.
			StartTimerGameTime(0.2, 1)
		EndIf

		; End this path test.
		Return
	EndIf

	; Get the current ingame game prior to running a path test
	Float time = Utility.GetCurrentGameTime()
	PathToReference(TestTarget, 0.25)

	; If the test was interrupted in fewer than 2 ingame minutes
	If (Utility.GetCurrentGameTime() - time < 0.033)
		; Delete the first follower if they exist
		If Follow1 != None
			Follow1.Delete()
		EndIf

		; Delete the first follower if they exist
		If Follow2 != None
			Follow2.Delete()
		EndIf

		; Delete this character
		Self.Delete()
	EndIf
EndFunction